const MIN_INTERVAL_MS = 1000;
const AUDIO_SLICE_INTERVAL = 1000;
const MAX_AUDIO_BUFFER_LEN = 32768;
const AUDIO_SAMPLE_RATE = 16000;

const API_URL = 'wss://api.developer.alt.ai/stt/stream'
const key = 'Your API Key';
const lang = 'ja-JP';

const socket = io(`${API_URL}?lang=${lang}&key=${key}`);

const input = document.getElementById("input");
const messages = document.getElementById("messages");

const audioContext = new AudioContext();

let recordLoopTimer;

const downSampling = (
  channelData,
  originalSampleRate,
  exportSampleRate
) => {
  const sampleRateRatio = originalSampleRate / exportSampleRate;
  const newDataLength = Math.round(channelData.length / sampleRateRatio);
  const result = new Float32Array(newDataLength);
  let offsetResult = 0;
  let offsetBuffer = 0;
  while (offsetResult < result.length) {
    const nextOffsetBuffer = Math.round(
      (offsetResult + 1) * sampleRateRatio
    );
    let accum = 0;
    let count = 0;
    for (
      let i = offsetBuffer;
      i < nextOffsetBuffer && i < channelData.length;
      i++
    ) {
      accum += channelData[i];
      count += 1;
    }
    result[offsetResult] = accum / count;
    offsetResult++;
    offsetBuffer = nextOffsetBuffer;
  }
  return result;
};

socket.on("message", (msg) => {
  console.log(msg);
  if (msg.status == 'RECOGNIZING') {
    console.log(msg.content)
  } if (msg.status == 'RECOGNIZED') {
    const item = document.createElement("li");
    item.textContent = `${msg.time}s: ${msg.content}`;
    messages.appendChild(item);
    window.scrollTo(0, document.body.scrollHeight);
  } else if (msg.status == 'CANCELED'){
    console.log('error');
    socket.disconnect();
  } else if (msg.status == 'SUCCEEDED') {
    console.log('finish');
    socket.disconnect();
  }
});

socket.on("key", (key) => {
  const item = document.createElement("li");
  item.textContent = "server received key: " + key;
  messages.appendChild(item);
  window.scrollTo(0, document.body.scrollHeight);
});

// collect DOMs
const display = document.querySelector(".display");
const controllerWrapper = document.querySelector(".controllers");

const State = ["Initial", "Record", "Download"];
let stateIndex = 0,
  totalSize = 0;
let mediaRecorder,
  chunks = [],
  audioURL = "",
  playbackChunks = [];

// mediaRecorder setup for audio
if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
  console.log("mediaDevices supported..");

  navigator.mediaDevices
    .getUserMedia({
      audio: true,
    })
    .then((stream) => {
      mediaRecorder = new MediaRecorder(stream);

      mediaRecorder.ondataavailable = async (e) => {
        chunks.push(e.data);
        console.log("data available! push chunks");
        playbackChunks.push(e.data);
      };
    })
    .catch((error) => {
      console.log("Following error has occured : ", error);
    });
} else {
  stateIndex = "";
  application(stateIndex);
}

const clearDisplay = () => {
  display.textContent = "";
};

const clearControls = () => {
  controllerWrapper.textContent = "";
};

const record = () => {
  if (socket.disconnected) {
    socket.connect()
  }
  stateIndex = 1;
  // mediaRecorder.start(MIN_INTERVAL_MS)
  application(stateIndex);
  console.log("start event");
  socket.emit("start");
  mediaRecorder.start(AUDIO_SLICE_INTERVAL);
  let blob = null
  let fr = null
  recordLoopTimer = setInterval(async () => {
    console.log("interval");
    mediaRecorder.stop();
    blob = null;
    blob = new Blob(chunks);
    if (blob.size > 0) {
      chunks = [];
      console.log('chunks: ', chunks)
      console.log(blob);
      mediaRecorder.start(AUDIO_SLICE_INTERVAL);
      if (blob !== null) {
        fr = null;
        fr = new FileReader();
        fr.onload = async () => {
          const arrayBuffer = fr.result;
          try {
            audioContext.decodeAudioData(
              arrayBuffer,
              async (audioBuffer) => {
                // if (this.isMuted) {
                //   return
                // }
                console.log(audioBuffer);
                const originalSampleRate = audioBuffer.sampleRate;
                const fistChannelIndex = 0;
                const f32Array =
                  audioBuffer.getChannelData(fistChannelIndex);
                console.log(f32Array)
                const modF32Array = downSampling(
                  f32Array,
                  originalSampleRate,
                  AUDIO_SAMPLE_RATE
                );
                const wavBlob = await encode({
                  sampleRate: AUDIO_SAMPLE_RATE,
                  channelData: [modF32Array],
                });
                console.log(wavBlob)

                socket.emit('audioChunk', wavBlob)
              }
            );
          } catch (error) {
            console.error("Unable to decode arraybuffer to audiobuffer.");
            console.error(error);
          }
        };
        fr.readAsArrayBuffer(blob);
      }
    }
  }, AUDIO_SLICE_INTERVAL);
};

const stopRecording = () => {
  mediaRecorder.stop();
  const blob = new Blob(playbackChunks, {
    type: "audio/ogg; codecs=opus",
  });
  playbackChunks = [];
  chunks = [];

  stateIndex = 2;
  application(stateIndex);

  audioURL = window.URL.createObjectURL(blob);
  document.querySelector("audio").src = audioURL;
  console.log("stop event");
  socket.emit("end");
  clearInterval(recordLoopTimer);
};

const downloadAudio = () => {
  const downloadLink = document.createElement("a");
  downloadLink.href = audioURL;
  downloadLink.setAttribute("download", "audio");
  downloadLink.click();
};

const addButton = (id, funString, text) => {
  const btn = document.createElement("button");
  btn.id = id;
  btn.setAttribute("onclick", funString);
  btn.textContent = text;
  controllerWrapper.append(btn);
};

const addMessage = (text) => {
  const msg = document.createElement("p");
  msg.textContent = text;
  display.append(msg);
};

const addAudio = () => {
  const audio = document.createElement("audio");
  audio.controls = true;
  audio.src = audioURL;
  display.append(audio);
};

const application = (index) => {
  switch (State[index]) {
    case "Initial":
      clearDisplay();
      clearControls();

      addButton("record", "record()", "Start Recording");
      break;

    case "Record":
      clearDisplay();
      clearControls();

      addMessage("Recording...");
      addButton("stop", "stopRecording()", "Stop Recording");
      break;

    case "Download":
      clearControls();
      clearDisplay();

      addAudio();
      addButton("record", "record()", "Record Again");
      break;

    default:
      clearControls();
      clearDisplay();

      addMessage("Your browser does not support mediaDevices");
      break;
  }
};

application(stateIndex);